@@ -756,7 +756,7 @@ float* GRIB2Section567Writer::GetFloatData()
         GDT_Float32,
         sizeof(float),
         m_adfGeoTransform[5] < 0 ?
-            static_cast<GSpacing>(-m_nXSize * sizeof(float)):
+            -static_cast<GSpacing>(m_nXSize * sizeof(float)):
             static_cast<GSpacing>(m_nXSize * sizeof(float)),
         nullptr);
     if( eErr != CE_None )
@@ -2110,18 +2110,19 @@ static float ComputeValOffset(int nTokens, char** papszTokens,
     float fValOffset = 0.0f;
 
     // Parameter category 0 = Temperature
-    bool bIsTemperature = false;
     if( nTokens >= 2 && atoi(papszTokens[0]) == 0 )
     {
-        // Cf http://www.nco.ncep.noaa.gov/pmb/docs/grib2/grib2_table4-2-0-0.shtml
+        // Cf https://www.nco.ncep.noaa.gov/pmb/docs/grib2/grib2_doc/grib2_table4-2-0-0.shtml
         // PARAMETERS FOR DISCIPLINE 0 CATEGORY 0
         int nParamNumber = atoi(papszTokens[1]);
-        if( nParamNumber >= 0 && nParamNumber <= 18 &&
+        if( (nParamNumber >= 0 && nParamNumber <= 18 &&
             nParamNumber != 8 && nParamNumber != 10 && nParamNumber != 11 &&
-            nParamNumber != 16 )
+            nParamNumber != 16) ||
+            nParamNumber == 21 ||
+            nParamNumber == 27 )
         {
-            bIsTemperature = true;
-            if( pszInputUnit == nullptr || EQUAL(pszInputUnit, "C") )
+            if( pszInputUnit == nullptr || EQUAL(pszInputUnit, "C") ||
+                EQUAL(pszInputUnit, "[C]") )
             {
                 fValOffset = 273.15f;
                 CPLDebug("GRIB",
@@ -2129,20 +2130,9 @@ static float ComputeValOffset(int nTokens, char** papszTokens,
                          "Celsius to Kelvin",
                          fValOffset);
             }
-            else if( !EQUAL(pszInputUnit, "K") )
-            {
-                CPLError(CE_Warning, CPLE_NotSupported,
-                            "Unsupported INPUT_UNIT = %s", pszInputUnit);
-            }
         }
     }
 
-    if( !bIsTemperature && pszInputUnit != nullptr )
-    {
-        CPLError(CE_Warning, CPLE_AppDefined,
-                 "INPUT_UNIT ignored for that product template");
-    }
-
     return fValOffset;
 }
 
@@ -2175,14 +2165,18 @@ static bool WriteSection4( VSILFILE* fp,
         pszPDSTemplateNumbers = GetBandOption(
             papszOptions, poSrcDS, nBand, "PDS_TEMPLATE_NUMBERS", nullptr);
     }
+    CPLString osInputUnit;
     const char* pszInputUnit = GetBandOption(
             papszOptions, nullptr, nBand, "INPUT_UNIT", nullptr);
     if( pszInputUnit == nullptr )
     {
         const char* pszGribUnit =
             poSrcDS->GetRasterBand(nBand)->GetMetadataItem("GRIB_UNIT");
-        if( pszGribUnit != nullptr && EQUAL(pszGribUnit, "[K]") )
-            pszInputUnit = "K";
+        if( pszGribUnit != nullptr )
+        {
+            osInputUnit = pszGribUnit;
+            pszInputUnit = osInputUnit.c_str();
+        }
     }
     WriteUInt16(fp, nPDTN); // PDTN
     if( nPDTN == 0 && pszPDSTemplateNumbers == nullptr &&
@@ -2394,75 +2388,6 @@ static bool WriteSection4( VSILFILE* fp,
     return true;
 }
 
-/************************************************************************/
-/*                            WriteBand()                               */
-/************************************************************************/
-
-static bool WriteBand( VSILFILE* fp, GDALDataset *poSrcDS, int nBand,
-                       char** papszOptions,
-                       GDALProgressFunc pfnProgress, void * pProgressData )
-{
-    // Section 0: Indicator section
-    vsi_l_offset nStartOffset = VSIFTellL(fp);
-    VSIFWriteL( "GRIB", 4, 1, fp );
-    WriteByte(fp, 0); // reserved
-    WriteByte(fp, 0); // reserved
-    int nDiscipline = atoi(GetBandOption(
-            papszOptions, poSrcDS, nBand, "DISCIPLINE", "0")); // 0 = Meteorological
-    WriteByte(fp, nDiscipline); // discipline
-    WriteByte(fp, 2); // GRIB edition number
-    vsi_l_offset nTotalSizeOffset = VSIFTellL(fp);
-    WriteUInt32(fp, GRIB2MISSING_u4); // dummy file size (high 32 bits)
-    WriteUInt32(fp, GRIB2MISSING_u4); // dummy file size (low 32 bits)
-
-    // Section 1: Identification Section
-    WriteSection1( fp, poSrcDS, nBand, papszOptions );
-
-    // Section 2: Local use section
-    WriteUInt32(fp, 5); // section size
-    WriteByte(fp, 2); // section number
-
-    // Section 3: Grid Definition Section
-    if( !GRIB2Section3Writer(fp, poSrcDS).Write() )
-    {
-        return false;
-    }
-
-    // Section 4: Product Definition Section
-    float fValOffset = 0.0f;
-    if( !WriteSection4( fp, poSrcDS, nBand, papszOptions, fValOffset ) )
-    {
-        return false;
-    }
-
-    // Section 5, 6 and 7
-    if( !GRIB2Section567Writer(fp, poSrcDS, nBand).
-            Write(fValOffset, papszOptions, pfnProgress, pProgressData) )
-    {
-        return false;
-    }
-
-    // Section 8: End section
-    VSIFWriteL( "7777", 4, 1, fp );
-
-    // Patch total message size at end of section 0
-    vsi_l_offset nCurOffset = VSIFTellL(fp);
-    if( nCurOffset - nStartOffset > INT_MAX )
-    {
-        CPLError(CE_Failure, CPLE_NotSupported,
-                 "GRIB message larger than 2 GB");
-        return false;
-    }
-    GUInt32 nTotalSize = static_cast<GUInt32>(nCurOffset - nStartOffset);
-    VSIFSeekL(fp, nTotalSizeOffset, SEEK_SET );
-    WriteUInt32(fp, 0); // file size (high 32 bits)
-    WriteUInt32(fp, nTotalSize); // file size (low 32 bits)
-
-    VSIFSeekL(fp, nCurOffset, SEEK_SET );
-
-    return true;
-}
-
 /************************************************************************/
 /*                             CreateCopy()                             */
 /************************************************************************/
@@ -2535,17 +2460,85 @@ GRIBDataset::CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
     }
     VSIFSeekL(fp, 0, SEEK_END);
 
-    for( int i = 0; i < poSrcDS->GetRasterCount(); i++ )
+    vsi_l_offset nStartOffset = 0;
+    vsi_l_offset nTotalSizeOffset = 0;
+    // Note: WRITE_SUBGRIDS=YES should not be used blindly currently, as it
+    // does not check that the content of the DISCIPLINE and IDS are the same.
+    // A smarter behavior would be to break into separate messages if needed
+    const bool bWriteSubGrids = CPLTestBool(CSLFetchNameValueDef(
+            papszOptions, "WRITE_SUBGRIDS", "NO"));
+    for( int nBand = 1; nBand <= poSrcDS->GetRasterCount(); nBand++ )
     {
-        if( !WriteBand( fp, poSrcDS, i + 1, papszOptions,
-                         pfnProgress, pProgressData ) )
+        if( nBand == 1 || !bWriteSubGrids )
+        {
+            // Section 0: Indicator section
+            nStartOffset = VSIFTellL(fp);
+            VSIFWriteL( "GRIB", 4, 1, fp );
+            WriteByte(fp, 0); // reserved
+            WriteByte(fp, 0); // reserved
+            int nDiscipline = atoi(GetBandOption(
+                    papszOptions, poSrcDS, nBand, "DISCIPLINE", "0")); // 0 = Meteorological
+            WriteByte(fp, nDiscipline); // discipline
+            WriteByte(fp, 2); // GRIB edition number
+            nTotalSizeOffset = VSIFTellL(fp);
+            WriteUInt32(fp, GRIB2MISSING_u4); // dummy file size (high 32 bits)
+            WriteUInt32(fp, GRIB2MISSING_u4); // dummy file size (low 32 bits)
+
+            // Section 1: Identification Section
+            WriteSection1( fp, poSrcDS, nBand, papszOptions );
+
+            // Section 2: Local use section
+            WriteUInt32(fp, 5); // section size
+            WriteByte(fp, 2); // section number
+
+            // Section 3: Grid Definition Section
+            if( !GRIB2Section3Writer(fp, poSrcDS).Write() )
+            {
+                VSIFCloseL(fp);
+                return nullptr;
+            }
+        }
+
+        // Section 4: Product Definition Section
+        float fValOffset = 0.0f;
+        if( !WriteSection4( fp, poSrcDS, nBand, papszOptions, fValOffset ) )
         {
             VSIFCloseL(fp);
             return nullptr;
         }
 
+        // Section 5, 6 and 7
+        if( !GRIB2Section567Writer(fp, poSrcDS, nBand).
+                Write(fValOffset, papszOptions, pfnProgress, pProgressData) )
+        {
+            VSIFCloseL(fp);
+            return nullptr;
+        }
+
+        if( nBand == poSrcDS->GetRasterCount() || !bWriteSubGrids )
+        {
+            // Section 8: End section
+            VSIFWriteL( "7777", 4, 1, fp );
+
+            // Patch total message size at end of section 0
+            vsi_l_offset nCurOffset = VSIFTellL(fp);
+            if( nCurOffset - nStartOffset > INT_MAX )
+            {
+                CPLError(CE_Failure, CPLE_NotSupported,
+                        "GRIB message larger than 2 GB");
+                VSIFCloseL(fp);
+                return nullptr;
+            }
+            GUInt32 nTotalSize = static_cast<GUInt32>(nCurOffset - nStartOffset);
+            VSIFSeekL(fp, nTotalSizeOffset, SEEK_SET );
+            WriteUInt32(fp, 0); // file size (high 32 bits)
+            WriteUInt32(fp, nTotalSize); // file size (low 32 bits)
+
+            VSIFSeekL(fp, nCurOffset, SEEK_SET );
+        }
+
         if( pfnProgress &&
-            !pfnProgress(static_cast<double>(i+1) / poSrcDS->GetRasterCount(),
+            !pfnProgress(static_cast<double>(nBand) / poSrcDS->GetRasterCount(),
                     nullptr, pProgressData ) )
         {
             VSIFCloseL(fp);
