@@ -6,7 +6,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2005, Frank Warmerdam
- * Copyright (c) 2009, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2009, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -56,7 +56,7 @@
 #endif
 #include "ogrgeojsonwriter.h"
 
-CPL_CVSID("$Id: gdal_rat.cpp ec7b85e6bb8f9737693a31f0bf7166e31e10992e 2018-04-16 00:08:36 +0200 Even Rouault $")
+CPL_CVSID("$Id: gdal_rat.cpp 63bfc5a68638c1bcb0efc729ca39e101ecf29691 2019-09-24 17:55:12 +0200 Even Rouault $")
 
 /**
  * \class GDALRasterAttributeTable
@@ -99,7 +99,7 @@ CPL_CVSID("$Id: gdal_rat.cpp ec7b85e6bb8f9737693a31f0bf7166e31e10992e 2018-04-16
 /*                      Virtual Destructor                              */
 /************************************************************************/
 
-GDALRasterAttributeTable::~GDALRasterAttributeTable() {}
+GDALRasterAttributeTable::~GDALRasterAttributeTable() = default;
 
 /************************************************************************/
 /*                              ValuesIO()                              */
@@ -556,6 +556,48 @@ GDALRATGetLinearBinning( GDALRasterAttributeTableH hRAT,
         GetLinearBinning( pdfRow0Min, pdfBinSize );
 }
 
+/************************************************************************/
+/*                        GDALRATGetTableType()                         */
+/************************************************************************/
+
+/**
+ * \brief Get Rat Table Type
+ *
+ * @since GDAL 2.4
+ *
+ * This function is the same as the C++ method GDALRasterAttributeTable::GetTableType()
+ */
+GDALRATTableType CPL_STDCALL
+GDALRATGetTableType( GDALRasterAttributeTableH hRAT)
+{
+    VALIDATE_POINTER1( hRAT, "GDALRATGetTableType", GRTT_THEMATIC );
+
+    return GDALDefaultRasterAttributeTable::FromHandle(hRAT)->
+        GetTableType();
+}
+
+/************************************************************************/
+/*                        GDALRATSetTableType()                         */
+/************************************************************************/
+
+/**
+ * \brief Set RAT Table Type
+ *
+ * @since GDAL 2.4
+ *
+ * This function is the same as the C++ method GDALRasterAttributeTable::SetTableType()
+ */
+CPLErr CPL_STDCALL
+GDALRATSetTableType( GDALRasterAttributeTableH hRAT,
+                         const GDALRATTableType eInTableType )
+
+{
+    VALIDATE_POINTER1( hRAT, "GDALRATSetTableType", CE_Failure );
+
+    return GDALDefaultRasterAttributeTable::FromHandle(hRAT)->
+        SetTableType( eInTableType );
+}
+
 /************************************************************************/
 /*                             Serialize()                              */
 /************************************************************************/
@@ -592,6 +634,22 @@ CPLXMLNode *GDALRasterAttributeTable::Serialize() const
             CXT_Text, szValue );
     }
 
+/* -------------------------------------------------------------------- */
+/*      Store table type                                                */
+/* -------------------------------------------------------------------- */
+    const GDALRATTableType tableType = GetTableType();
+    if (tableType == GRTT_ATHEMATIC)
+    {
+        CPLsnprintf( szValue, sizeof(szValue), "athematic" );
+    }
+    else
+    {
+        CPLsnprintf( szValue, sizeof(szValue), "thematic" );
+    }
+    CPLCreateXMLNode(
+        CPLCreateXMLNode( psTree, CXT_Attribute, "tableType" ),
+        CXT_Text, szValue );
+
 /* -------------------------------------------------------------------- */
 /*      Define each column.                                             */
 /* -------------------------------------------------------------------- */
@@ -682,6 +740,7 @@ void *GDALRasterAttributeTable::SerializeJSON() const
     double dfBinSize = 0.0;
     json_object *poRow0Min = nullptr;
     json_object *poBinSize = nullptr;
+    json_object *poTableType = nullptr;
 
     if( GetLinearBinning(&dfRow0Min, &dfBinSize) )
     {
@@ -692,6 +751,20 @@ void *GDALRasterAttributeTable::SerializeJSON() const
         json_object_object_add( poRAT, "binSize", poBinSize );
     }
 
+/* -------------------------------------------------------------------- */
+/*      Table Type                                                      */
+/* -------------------------------------------------------------------- */
+    const GDALRATTableType tableType = GetTableType();
+    if (tableType == GRTT_ATHEMATIC)
+    {
+        poTableType = json_object_new_string( "athematic" );
+    }
+    else
+    {
+        poTableType = json_object_new_string( "thematic" );
+    }
+    json_object_object_add( poRAT, "tableType", poTableType);
+
 /* -------------------------------------------------------------------- */
 /*      Define each column.                                             */
 /* -------------------------------------------------------------------- */
@@ -782,6 +855,23 @@ CPLErr GDALRasterAttributeTable::XMLInit( CPLXMLNode *psTree,
                           CPLAtof(CPLGetXMLValue( psTree, "BinSize","" )) );
     }
 
+/* -------------------------------------------------------------------- */
+/*      Table Type                                                      */
+/* -------------------------------------------------------------------- */
+    if( CPLGetXMLValue( psTree, "tableType", nullptr ) )
+    {
+        const char* pszValue = CPLGetXMLValue(psTree, "tableType", "thematic");
+        if (EQUAL(pszValue, "athematic"))
+        {
+            SetTableType(GRTT_ATHEMATIC);
+        }
+        else
+        {
+            SetTableType(GRTT_THEMATIC);
+        }
+    }
+
+
 /* -------------------------------------------------------------------- */
 /*      Column definitions                                              */
 /* -------------------------------------------------------------------- */
@@ -964,8 +1054,10 @@ GDALColorTable *GDALRasterAttributeTable::TranslateToColorTable(
             return nullptr;
 
         for( int iRow = 0; iRow < GetRowCount(); iRow++ )
+        {
             nEntryCount =
-                std::max(nEntryCount, GetValueAsInt(iRow, iMaxCol) + 1);
+                std::max(nEntryCount, std::min(65535, GetValueAsInt(iRow, iMaxCol)) + 1);
+        }
 
         if( nEntryCount < 0 )
             return nullptr;
@@ -1091,6 +1183,7 @@ GDALDefaultRasterAttributeTable::GDALDefaultRasterAttributeTable() :
     bLinearBinning(false),
     dfRow0Min(-0.5),
     dfBinSize(1.0),
+    eTableType(GRTT_THEMATIC),
     bColumnsAnalysed(false),
     nMinCol(-1),
     nMaxCol(-1),
@@ -1113,27 +1206,13 @@ GDALRasterAttributeTableH CPL_STDCALL GDALCreateRasterAttributeTable()
     return new GDALDefaultRasterAttributeTable();
 }
 
-/************************************************************************/
-/*                  GDALDefaultRasterAttributeTable()                   */
-/************************************************************************/
-
-//! Copy constructor.
-
-GDALDefaultRasterAttributeTable::GDALDefaultRasterAttributeTable(
-    const GDALDefaultRasterAttributeTable &oOther ) : GDALRasterAttributeTable()
-
-{
-    // We have tried to be careful to allow wholesale assignment
-    *this = oOther;
-}
-
 /************************************************************************/
 /*                 ~GDALDefaultRasterAttributeTable()                   */
 /*                                                                      */
 /*      All magic done by magic by the container destructors.           */
 /************************************************************************/
 
-GDALDefaultRasterAttributeTable::~GDALDefaultRasterAttributeTable() {}
+GDALDefaultRasterAttributeTable::~GDALDefaultRasterAttributeTable() = default;
 
 /************************************************************************/
 /*                  GDALDestroyRasterAttributeTable()                   */
@@ -1592,20 +1671,20 @@ void GDALDefaultRasterAttributeTable::SetRowCount( int nNewCount )
     if( nNewCount == nRowCount )
         return;
 
-    for( unsigned int iField = 0; iField < aoFields.size(); iField++ )
+    for( auto& oField: aoFields )
     {
-        switch( aoFields[iField].eType )
+        switch( oField.eType )
         {
           case GFT_Integer:
-            aoFields[iField].anValues.resize( nNewCount );
+            oField.anValues.resize( nNewCount );
             break;
 
           case GFT_Real:
-            aoFields[iField].adfValues.resize( nNewCount );
+            oField.adfValues.resize( nNewCount );
             break;
 
           case GFT_String:
-            aoFields[iField].aosValues.resize( nNewCount );
+            oField.aosValues.resize( nNewCount );
             break;
         }
     }
@@ -1978,6 +2057,52 @@ int GDALDefaultRasterAttributeTable::GetLinearBinning(
     return true;
 }
 
+/************************************************************************/
+/*                          GetTableType()                              */
+/************************************************************************/
+
+/**
+ * \brief Get RAT Table Type
+ *
+ * Returns whether table type is thematic or athematic
+ *
+ * This method is the same as the C function GDALRATGetTableType().
+ *
+ * @since GDAL 2.4
+ *
+ * @return GRTT_THEMATIC or GRTT_ATHEMATIC
+ */
+
+GDALRATTableType GDALDefaultRasterAttributeTable::GetTableType() const
+{
+    return eTableType;
+}
+
+/************************************************************************/
+/*                          SetTableType()                              */
+/************************************************************************/
+
+/**
+ * \brief Set RAT Table Type
+ *
+ * Set whether table type is thematic or athematic
+ *
+ * This method is the same as the C function GDALRATSetTableType().
+ *
+ * @param eInTableType the new RAT table type (GRTT_THEMATIC or GRTT_ATHEMATIC)
+ *
+ * @since GDAL 2.4
+ *
+ * @return CE_None on success or CE_Failure on failure.
+ */
+
+CPLErr GDALDefaultRasterAttributeTable::SetTableType(const GDALRATTableType eInTableType)
+{
+    eTableType = eInTableType;
+    return CE_None;
+}
+
+
 /************************************************************************/
 /*                            CreateColumn()                            */
 /************************************************************************/
@@ -2013,6 +2138,57 @@ CPLErr GDALDefaultRasterAttributeTable::CreateColumn(
     return CE_None;
 }
 
+/************************************************************************/
+/*                            RemoveStatistics()                        */
+/************************************************************************/
+
+/**
+ * \brief Remove Statistics from RAT
+ *
+ * Remove statistics (such as histogram) from the RAT. This is important
+ * if these have been invalidated, for example by cropping the image.
+ *
+ * This method is the same as the C function GDALRATRemoveStatistics().
+ *
+ * @since GDAL 2.4
+ */
+
+void GDALDefaultRasterAttributeTable::RemoveStatistics()
+
+{
+    // since we are storing the fields in a vector it will generally
+    // be faster to create a new vector and replace the old one
+    // rather than actually erasing columns.
+    std::vector<GDALRasterAttributeField> aoNewFields;
+    for ( const auto& field : aoFields )
+    {
+        switch (field.eUsage)
+        {
+            case GFU_PixelCount:
+            case GFU_Min:
+            case GFU_Max:
+            case GFU_RedMin:
+            case GFU_GreenMin:
+            case GFU_BlueMin:
+            case GFU_AlphaMin:
+            case GFU_RedMax:
+            case GFU_GreenMax:
+            case GFU_BlueMax:
+            case GFU_AlphaMax:
+            {
+                break;
+            }
+
+            default:
+                if (field.sName != "Histogram")
+                {
+                    aoNewFields.push_back(field);
+                }
+        }
+    }
+    aoFields = aoNewFields;
+}
+
 /************************************************************************/
 /*                               Clone()                                */
 /************************************************************************/
@@ -2033,7 +2209,7 @@ GDALDefaultRasterAttributeTable *GDALDefaultRasterAttributeTable::Clone() const
  * This function is the same as the C++ method GDALRasterAttributeTable::Clone()
  */
 GDALRasterAttributeTableH CPL_STDCALL
-GDALRATClone( GDALRasterAttributeTableH hRAT )
+GDALRATClone( const GDALRasterAttributeTableH hRAT )
 
 {
     VALIDATE_POINTER1( hRAT, "GDALRATClone", nullptr );
@@ -2059,3 +2235,24 @@ GDALRATSerializeJSON( GDALRasterAttributeTableH hRAT )
 
     return GDALRasterAttributeTable::FromHandle(hRAT)->SerializeJSON();
 }
+
+
+/************************************************************************/
+/*                        GDALRATRemoveStatistics()                     */
+/************************************************************************/
+
+/**
+ * \brief Remove Statistics from RAT
+ *
+ * This function is the same as the C++ method GDALRasterAttributeTable::RemoveStatistics()
+ *
+ * @since GDAL 2.4
+ */
+void CPL_STDCALL
+GDALRATRemoveStatistics( GDALRasterAttributeTableH hRAT )
+
+{
+    VALIDATE_POINTER0( hRAT, "GDALRATRemoveStatistics" );
+
+    GDALRasterAttributeTable::FromHandle(hRAT)->RemoveStatistics();
+}
