@@ -2,10 +2,10 @@
  *
  * Project:  BNA Translator
  * Purpose:  Implements OGRBNALayer class.
- * Author:   Even Rouault, even dot rouault at mines dash paris dot org
+ * Author:   Even Rouault, even dot rouault at spatialys.com
  *
  ******************************************************************************
- * Copyright (c) 2007-2010, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2007-2010, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -32,7 +32,7 @@
 #include "cpl_csv.h"
 #include "ogr_p.h"
 
-CPL_CVSID("$Id: ogrbnalayer.cpp f9af14bc04e6f4fea8175f0e212455c2b6520378 2018-06-01 14:24:48 +0200 Even Rouault $")
+CPL_CVSID("$Id: ogrbnalayer.cpp f459e96d58bcda014371ee8b511d27e3c16b3152 2019-10-20 15:34:30 +0200 Even Rouault $")
 
 /************************************************************************/
 /*                            OGRBNALayer()                             */
@@ -55,9 +55,7 @@ OGRBNALayer::OGRBNALayer( const char *pszFilename,
     failed(false),
     curLine(0),
     nNextFID(0),
-    nFeatures(0),
-    partialIndexTable(TRUE),
-    offsetAndLineFeaturesTable(nullptr)
+    nFeatures(0)
 {
     static const char* const iKnowHowToCount[]
         = { "Primary", "Secondary", "Third", "Fourth", "Fifth" };
@@ -119,8 +117,6 @@ OGRBNALayer::~OGRBNALayer()
 {
     poFeatureDefn->Release();
 
-    CPLFree(offsetAndLineFeaturesTable);
-
     if (fpBNA)
         VSIFCloseL( fpBNA );
 }
@@ -128,14 +124,10 @@ OGRBNALayer::~OGRBNALayer()
 /************************************************************************/
 /*                         SetFeatureIndexTable()                       */
 /************************************************************************/
-void OGRBNALayer::SetFeatureIndexTable(
-    int nFeaturesIn,
-    OffsetAndLine* offsetAndLineFeaturesTableIn,
-    int partialIndexTableIn )
+void OGRBNALayer::SetFeatureIndexTable(std::vector<OffsetAndLine>&& offsetAndLineFeaturesTableIn)
 {
-    nFeatures = nFeaturesIn;
-    offsetAndLineFeaturesTable = offsetAndLineFeaturesTableIn;
-    partialIndexTable = CPL_TO_BOOL(partialIndexTableIn);
+    nFeatures = static_cast<int>(offsetAndLineFeaturesTableIn.size());
+    offsetAndLineFeaturesTable = std::move(offsetAndLineFeaturesTableIn);
 }
 
 /************************************************************************/
@@ -160,21 +152,16 @@ void OGRBNALayer::ResetReading()
 
 OGRFeature *OGRBNALayer::GetNextFeature()
 {
-    if( failed || eof || fpBNA == nullptr )
+    if( failed || eof || fpBNA == nullptr  )
         return nullptr;
 
-    while( true )
+    while( nNextFID < nFeatures )
     {
         int ok = FALSE;
-        const int offset = static_cast<int>( VSIFTellL(fpBNA) );
-        const int line = curLine;
-        if (nNextFID < nFeatures)
-        {
-            if( VSIFSeekL( fpBNA, offsetAndLineFeaturesTable[nNextFID].offset,
-                           SEEK_SET ) < 0 )
-                return nullptr;
-            curLine = offsetAndLineFeaturesTable[nNextFID].line;
-        }
+        if( VSIFSeekL( fpBNA, offsetAndLineFeaturesTable[nNextFID].offset,
+                        SEEK_SET ) < 0 )
+            return nullptr;
+        curLine = offsetAndLineFeaturesTable[nNextFID].line;
         BNARecord* record
             = BNA_GetNextRecord(fpBNA, &ok, &curLine, TRUE, bnaFeatureType);
         if (ok == FALSE)
@@ -187,25 +174,11 @@ OGRFeature *OGRBNALayer::GetNextFeature()
         {
             /* end of file */
             eof = true;
-
-            /* and we have finally build the whole index table */
-            partialIndexTable = false;
             return nullptr;
         }
 
         if (record->featureType == bnaFeatureType)
         {
-            if (nNextFID >= nFeatures)
-            {
-                nFeatures++;
-                offsetAndLineFeaturesTable =
-                    static_cast<OffsetAndLine *>(
-                        CPLRealloc( offsetAndLineFeaturesTable,
-                                    nFeatures * sizeof(OffsetAndLine) ) );
-                offsetAndLineFeaturesTable[nFeatures-1].offset = offset;
-                offsetAndLineFeaturesTable[nFeatures-1].line = line;
-            }
-
             OGRFeature *poFeature
                 = BuildFeatureFromBNARecord(record, nNextFID++);
 
@@ -226,6 +199,7 @@ OGRFeature *OGRBNALayer::GetNextFeature()
             BNA_FreeRecord(record);
         }
     }
+    return nullptr;
 }
 
 /************************************************************************/
@@ -601,10 +575,7 @@ OGRFeature *OGRBNALayer::BuildFeatureFromBNARecord (BNARecord* record, long fid)
         int isFirstPolygon = 1;
         double secondaryFirstX = 0.0;
         double secondaryFirstY = 0.0;
-
-        OGRLinearRing* ring = new OGRLinearRing ();
-        ring->setCoordinateDimension(2);
-        ring->addPoint(record->tabCoords[0][0], record->tabCoords[0][1] );
+        int iStartIndex = 0;
 
         /* record->nCoords is really a safe upper bound */
         int nbPolygons = 0;
@@ -615,14 +586,21 @@ OGRFeature *OGRBNALayer::BuildFeatureFromBNARecord (BNARecord* record, long fid)
         int i = 1;
         for( ; i < record->nCoords; i++ )
         {
-            ring->addPoint(record->tabCoords[i][0], record->tabCoords[i][1] );
             if (isFirstPolygon == 1 &&
                 record->tabCoords[i][0] == firstX &&
                 record->tabCoords[i][1] == firstY)
             {
                 OGRPolygon* polygon = new OGRPolygon ();
+                OGRLinearRing* ring = new OGRLinearRing ();
+                ring->setCoordinateDimension(2);
+                ring->setNumPoints(i - iStartIndex + 1, false);
+                for( int j = iStartIndex; j <= i ; ++j )
+                {
+                    ring->setPoint( j - iStartIndex,
+                                    record->tabCoords[j][0],
+                                    record->tabCoords[j][1] );
+                }
                 polygon->addRingDirectly(ring);
-                ring = nullptr;
                 tabPolygons[nbPolygons] = polygon;
                 nbPolygons++;
 
@@ -636,10 +614,7 @@ OGRFeature *OGRBNALayer::BuildFeatureFromBNARecord (BNARecord* record, long fid)
                 i++;
                 secondaryFirstX = record->tabCoords[i][0];
                 secondaryFirstY = record->tabCoords[i][1];
-                ring = new OGRLinearRing ();
-                ring->setCoordinateDimension(2);
-                ring->addPoint( record->tabCoords[i][0],
-                                record->tabCoords[i][1] );
+                iStartIndex = i;
             }
             else if (isFirstPolygon == 0 &&
                     record->tabCoords[i][0] == secondaryFirstX &&
@@ -647,10 +622,31 @@ OGRFeature *OGRBNALayer::BuildFeatureFromBNARecord (BNARecord* record, long fid)
             {
 
                 OGRPolygon* polygon = new OGRPolygon ();
+                OGRLinearRing* ring = new OGRLinearRing ();
+                ring->setCoordinateDimension(2);
+                ring->setNumPoints(i - iStartIndex + 1, false);
+                for( int j = iStartIndex; j <= i ; ++j )
+                {
+                    ring->setPoint( j - iStartIndex,
+                                    record->tabCoords[j][0],
+                                    record->tabCoords[j][1] );
+                }
                 polygon->addRingDirectly(ring);
-                ring = nullptr;
-                tabPolygons[nbPolygons] = polygon;
-                nbPolygons++;
+
+                for( int j = 0; j < nbPolygons; j++ )
+                {
+                    if( polygon->Equals(tabPolygons[j]) )
+                    {
+                        delete polygon;
+                        polygon = nullptr;
+                        break;
+                    }
+                }
+                if( polygon )
+                {
+                    tabPolygons[nbPolygons] = polygon;
+                    nbPolygons++;
+                }
 
                 if (i < record->nCoords - 1)
                 {
@@ -679,10 +675,7 @@ OGRFeature *OGRBNALayer::BuildFeatureFromBNARecord (BNARecord* record, long fid)
                     i ++;
                     secondaryFirstX = record->tabCoords[i][0];
                     secondaryFirstY = record->tabCoords[i][1];
-                    ring = new OGRLinearRing ();
-                    ring->setCoordinateDimension(2);
-                    ring->addPoint( record->tabCoords[i][0],
-                                    record->tabCoords[i][1] );
+                    iStartIndex = i;
                 }
                 else
                 {
@@ -702,17 +695,28 @@ OGRFeature *OGRBNALayer::BuildFeatureFromBNARecord (BNARecord* record, long fid)
             /* Let's be a bit tolerant about non-closing polygons. */
             if (isFirstPolygon)
             {
-                ring->addPoint( record->tabCoords[0][0],
+                CPLAssert( iStartIndex == 0 );
+                OGRLinearRing* ring = new OGRLinearRing ();
+                ring->setCoordinateDimension(2);
+                ring->setNumPoints(record->nCoords + 1, false);
+                for( int j = 0; j < record->nCoords ; ++j )
+                {
+                    ring->setPoint( j,
+                                    record->tabCoords[j][0],
+                                    record->tabCoords[j][1] );
+                }
+
+                ring->setPoint( record->nCoords,
+                                record->tabCoords[0][0],
                                 record->tabCoords[0][1] );
 
                 OGRPolygon* polygon = new OGRPolygon ();
                 polygon->addRingDirectly(ring);
-                ring = nullptr;
+
                 tabPolygons[nbPolygons] = polygon;
                 nbPolygons++;
             }
         }
-        delete ring;
 
         if (nbPolygons == 1)
         {
@@ -772,100 +776,28 @@ OGRFeature *OGRBNALayer::BuildFeatureFromBNARecord (BNARecord* record, long fid)
     return poFeature;
 }
 
-/************************************************************************/
-/*                           FastParseUntil()                           */
-/************************************************************************/
-void OGRBNALayer::FastParseUntil ( int interestFID)
-{
-    if( partialIndexTable )
-    {
-        ResetReading();
-
-        BNARecord *record = nullptr;
-
-        if (nFeatures > 0)
-        {
-            if( VSIFSeekL( fpBNA,
-                           offsetAndLineFeaturesTable[nFeatures-1].offset,
-                           SEEK_SET ) < 0 )
-                return;
-            curLine = offsetAndLineFeaturesTable[nFeatures-1].line;
-
-            /* Just skip the last read one */
-            int ok = FALSE;
-            record = BNA_GetNextRecord(fpBNA, &ok, &curLine, TRUE,
-                                       BNA_READ_NONE);
-            BNA_FreeRecord(record);
-        }
-
-        while(1)
-        {
-            int ok = FALSE;
-            int offset = static_cast<int>( VSIFTellL(fpBNA) );
-            int line = curLine;
-            record = BNA_GetNextRecord(fpBNA, &ok, &curLine, TRUE,
-                                       BNA_READ_NONE);
-            if (ok == FALSE)
-            {
-                failed = true;
-                return;
-            }
-            if (record == nullptr)
-            {
-                /* end of file */
-                eof = true;
-
-                /* and we have finally build the whole index table */
-                partialIndexTable = false;
-                return;
-            }
-
-            if (record->featureType == bnaFeatureType)
-            {
-                nFeatures++;
-                offsetAndLineFeaturesTable =
-                    static_cast<OffsetAndLine *>(
-                        CPLRealloc( offsetAndLineFeaturesTable,
-                                    nFeatures * sizeof(OffsetAndLine) ) );
-                offsetAndLineFeaturesTable[nFeatures-1].offset = offset;
-                offsetAndLineFeaturesTable[nFeatures-1].line = line;
-
-                BNA_FreeRecord(record);
-
-                if (nFeatures - 1 == interestFID)
-                  return;
-            }
-            else
-            {
-                BNA_FreeRecord(record);
-            }
-        }
-    }
-}
-
 /************************************************************************/
 /*                           GetFeature()                               */
 /************************************************************************/
 
 OGRFeature *  OGRBNALayer::GetFeature( GIntBig nFID )
 {
-    if (nFID < 0 || !CPL_INT64_FITS_ON_INT32(nFID))
+    if (bWriter || nFID < 0 || !CPL_INT64_FITS_ON_INT32(nFID))
         return nullptr;
 
-    FastParseUntil( static_cast<int>( nFID ) );
-
+    const int nFID32 = static_cast<int>(nFID);
     if (nFID >= nFeatures)
         return nullptr;
 
-    if( VSIFSeekL( fpBNA, offsetAndLineFeaturesTable[nFID].offset, SEEK_SET ) < 0 )
+    if( VSIFSeekL( fpBNA, offsetAndLineFeaturesTable[nFID32].offset, SEEK_SET ) < 0 )
         return nullptr;
 
-    curLine = offsetAndLineFeaturesTable[nFID].line;
+    curLine = offsetAndLineFeaturesTable[nFID32].line;
     int ok = FALSE;
     BNARecord* record
         = BNA_GetNextRecord(fpBNA, &ok, &curLine, TRUE, bnaFeatureType);
 
-    OGRFeature *poFeature = BuildFeatureFromBNARecord(record, (int)nFID);
+    OGRFeature *poFeature = BuildFeatureFromBNARecord(record, nFID32);
 
     BNA_FreeRecord(record);
 
