@@ -7,7 +7,7 @@
 # Howard Butler hobu.inc@gmail.com
 
 
-gdal_version = '3.0.2'
+gdal_version = '3.2.1'
 
 import sys
 import os
@@ -240,6 +240,7 @@ class gdal_ext(build_ext):
         self.gdaldir = None
         self.gdal_config = self.GDAL_CONFIG
         self.extra_cflags = []
+        self.parallel = True # Python 3.5 only
 
     def get_compiler(self):
         return self.compiler or get_default_compiler()
@@ -252,7 +253,15 @@ class gdal_ext(build_ext):
             # the gdal-config location given in setup.cfg is
             # incorrect, or possibly the default -- ../../apps/gdal-config
             # We'll try to use the gdal-config that might be on the path.
-            return fetch_config(option)
+            try:
+                return fetch_config(option)
+            except gdal_config_error as e:
+                msg = 'Could not find gdal-config. Make sure you have installed the GDAL native library and development headers.'
+                import sys
+                import traceback
+                traceback_string = ''.join(traceback.format_exception(*sys.exc_info()))
+                raise gdal_config_error(traceback_string + '\n' + msg)
+
 
     def build_extensions(self):
 
@@ -323,6 +332,24 @@ class gdal_ext(build_ext):
         ext.extra_compile_args.extend(self.extra_cflags)
         return build_ext.build_extension(self, ext)
 
+# This is only needed with Python 2.
+if sys.version_info < (3,):
+    try:
+        import multiprocessing
+        from concurrent.futures import ThreadPoolExecutor as Pool
+
+        num_jobs = multiprocessing.cpu_count()
+
+        def parallel_build_extensions(self):
+            self.check_extensions_list(self.extensions)
+
+            with Pool(num_jobs) as pool:
+                # Note: map() returns an iterator that needs to be consumed.
+                list(pool.map(self.build_extension, self.extensions))
+
+        build_ext.build_extensions = parallel_build_extensions
+    except:
+        pass
 
 extra_link_args = []
 extra_compile_args = []
@@ -363,11 +390,6 @@ ext_modules = [gdal_module,
                osr_module,
                ogr_module]
 
-py_modules = ['gdal',
-              'ogr',
-              'osr',
-              'gdalconst']
-
 if os.path.exists('setup_vars.ini'):
     with open('setup_vars.ini') as f:
         lines = f.readlines()
@@ -379,9 +401,8 @@ if GNM_ENABLED:
 
 if HAVE_NUMPY:
     ext_modules.append(array_module)
-    py_modules.append('gdalnumeric')
 
-packages = ["osgeo", ]
+packages = ["osgeo", "osgeo.utils"]
 
 readme = str(open('README.rst', 'rb').read())
 
@@ -431,13 +452,13 @@ setup_kwargs = dict(
     description=description,
     license=license_type,
     classifiers=classifiers,
-    py_modules=py_modules,
     packages=packages,
     url=url,
     data_files=data_files,
     ext_modules=ext_modules,
     scripts=glob('scripts/*.py'),
-    cmdclass={'build_ext': gdal_ext}
+    cmdclass={'build_ext': gdal_ext},
+    extras_require={'numpy': ['numpy > 1.0.0']},
 )
 
 # This section can be greatly simplified with python >= 3.5 using **
